/*
   Rename imports.

   This script renames entries of a dynamically built import table.
   For example, from a table like this:

      dd offset ntdll_NtPowerInformation
      dd offset ntdll_NtInitiatePowerAction
      dd offset ntdll_NtSetThreadExecutionState
      dd offset ntdll_NtRequestWakeupLatency
      dd offset ntdll_NtGetDevicePowerState
      dd offset ntdll_NtIsSystemResumeAutomatic
      dd offset ntdll_NtRequestDeviceWakeup
      dd offset ntdll_NtCancelDeviceWakeupRequest
      dd offset ntdll_RtlQueryRegistryValues


   it will create a table like this:

      NtPowerInformation dd offset ntdll_NtPowerInformation
      NtInitiatePowerAction dd offset ntdll_NtInitiatePowerAction
      NtSetThreadExecutionState dd offset ntdll_NtSetThreadExecutionState
      NtRequestWakeupLatency dd offset ntdll_NtRequestWakeupLatency
      NtGetDevicePowerState dd offset ntdll_NtGetDevicePowerState
      NtIsSystemResumeAutomatic dd offset ntdll_NtIsSystemResumeAutomatic
      NtRequestDeviceWakeup dd offset ntdll_NtRequestDeviceWakeup
      NtCancelDeviceWakeupRequest dd offset ntdll_NtCancelDeviceWakeupRequest
      RtlQueryRegistryValues dd offset ntdll_RtlQueryRegistryValues

   Usage: select the import table and run the script.

   Known problems: if the dll name contains an underscore, the function
   names might be incorrect. Special care is taken for the ws2_32.dll but
   other dlls will have wrong function names.

*/

#include <idc.idc>

static main()
{
  auto ea1, ea2, idx, dllname, name;

  ea1 = SelStart();
  ea2 = SelEnd();
  if ( ea1 == BADADDR )
  {
    Warning("Please select the import table before running the renimp script");
  }
  else
  {
    while ( ea1 < ea2 )
    {
      name = Name(Dword(ea1));
      idx = strstr(name, "_");
      dllname = substr(name, 0, idx);
      if ( dllname == "ws2" )   // most likely the dll name is ws2_32
        idx = idx + 3;
      // extract the function name
      name = substr(name, idx+1, -1);
      if ( !MakeNameEx(ea1, name, SN_CHECK|SN_NOWARN) )
      {
        // failed to give a name - it could be that the name has already been
        // used in the program. add a suffix
        for ( idx=0; idx < 99; idx++ )
          if ( MakeNameEx(ea1, name + "_" + ltoa(idx, 10), SN_CHECK|SN_NOWARN) )
            break;
      }
      ea1 = ea1 + 4;
    }
  }
}